--[[--------------------------------------------------------------------------------------------------
ScoreHandler

Version: 		002
Author: 		Torben Ratzlaff
Website: 		screenheadmedia.com

Purpose:
Administrates highscores via predefined functions.

----------------------------------------------------------------------------------------------------]]


--[[--------------------------------------------------------------------------------------------------
How to use:

--defines a predefined avaluation function for a specific score value
ScoreHandler.setScoreAction(name, functionName)
--name 				- name of property (also used to identify the value in the score table)
--functionName 		- name of the used evaluation function. Allready defined functions are: "total", "highest", "lowest", "highestRow", "lowestRow"


--sets multiple score actions at once. uses ScoreHandler.setScoreAction()
ScoreHandler.setScoreActionSet(dataSet)
--dataSet			- a table containing several tables with the following fields: {name, functionName}. See ScoreHandler.setScoreAction() for more infos.


--defines a function that is saved under a given name
ScoreHandler.setScoreFunction(name, scoreFunction)
--functionName 		- name of the used evaluation function
--scoreFunction 	- the function that evaluates the values. 
					  New functions must be specified as following:

							function exampleFunction(action, currentValue, newValue)
								if action == "set" then
									--perform evaluation actions
									return valueToSet
								else
									--perform evaluation actions
									return valueToGet
								end
							end
							--action 		- the action to take ("set" or "get")
							--currentValue 	- the current value of the score
							--newValue 		- the new value that should be evaluated against the current one
							--return 		- the value that should be set or get


--sets the score with an action defined by ScoreHandler.setScoreAction()
ScoreHandler.setScore(name, setValue)
--name 				- name of the value
--setValue 			- the new value that should be set / evaluated
--return 			- return the set value. returns also true as a second value, if the new value is better (depending on the evaluation function) then the old one. Otherwise returns false as second parameter.


--gets the score with an action defined by ScoreHandler.setScoreAction()
ScoreHandler.getScore(name)
--name 				- name of the value


--sets the scoreTable that is used to store and retrieve scores
ScoreHandler.setScoreTable(scoreTable)
--scoreTable 		- table that contains the scores


-sets te default path to all scores inside the scoreTable. 
ScoreHandler.setDefaultScorePath(path)
--path 				- path of the score values. Default is nil.

----------------------------------------------------------------------------------------------------]]


local table_remove = table.remove


local function totalValue(action, currentValue, newValue)
	if action == "set" then
		return currentValue + newValue
	else
		return currentValue
	end
end

local function highestValue(action, currentValue, newValue)
	if action == "set" then
		return newValue > currentValue and newValue or currentValue
	else
		return currentValue
	end
end

local function lowestValue(action, currentValue, newValue)
	if action == "set" then
		return newValue < currentValue and newValue or currentValue
	else
		return currentValue
	end
end

local function highestRowValue(action, currentValue, newValue)
	if action == "set" then
		table_remove(currentValue, 1)
		currentValue[#currentValue + 1] = newValue

		return currentValue
	else
		local highestValue = currentValue[1]
		
		for i=2, #currentValue do
			local checkValue = currentValue[i]
			
			if checkValue < highestValue then highestValue = checkValue end
		end
		
		return highestValue
	end	
end

local function lowestRowValue(action, currentValue, newValue)
	if action == "set" then
		table_remove(currentValue, 1)
		currentValue[#currentValue + 1] = newValue

		return currentValue
	else
		local lowestValue = currentValue[1]
		
		for i=2, #currentValue do
			local checkValue = currentValue[i]
			
			if checkValue > lowestValue then lowestValue = checkValue end
		end
		
		return lowestValue
	end	
end


local ScoreHandler = {}

local scoreFunctions = {
	total 		= totalValue,
	highest 	= highestValue,
	lowest 		= lowestValue,
	highestRow 	= highestRowValue,
	lowestRow 	= lowestRowValue,
}

local scoreActions = {}
local defaultScoreTable = {}
local currentScoreTable = defaultScoreTable
local currentScoreSubTable = defaultScoreTable
local defaultScorePath


function ScoreHandler.setScoreAction(name, functionName)
	scoreActions[name] = functionName
end


function ScoreHandler.setScoreFunction(name, scoreFunction)
	scoreFunctions[name] = scoreFunction
end


function ScoreHandler.setScore(name, setValue)
	local currentValue = currentScoreSubTable[name]
	local newValue 

	if currentValue then
		newValue = scoreFunctions[scoreActions[name]]("set", currentValue, setValue)
	else
		newValue = setValue
	end
		
	if newValue and newValue ~= currentValue then
		currentScoreSubTable[name] = newValue

		return newValue, true
	end

	return newValue, false
end


function ScoreHandler.getScore(name)
	return scoreFunctions[scoreActions[name]]("get", currentScoreSubTable[name])
end


function ScoreHandler.setScoreTable(scoreTable)
	currentScoreTable = scoreTable

	if defaultScorePath then
		currentScoreSubTable = currentScoreTable[defaultScorePath]
	else
		currentScoreSubTable = currentScoreTable
	end
end

function ScoreHandler.setScorePath(path)
	defaultScorePath = path

	if defaultScorePath then
		currentScoreSubTable = currentScoreTable[defaultScorePath]
	else
		currentScoreSubTable = currentScoreTable
	end
end


return ScoreHandler